<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\Migrations\Main;

use Doctrine\DBAL\Schema\Schema;
use Doctrine\Migrations\AbstractMigration;

final class Version20250214154310 extends AbstractMigration
{
    public function getDescription(): string
    {
        return 'Create view for searching address reference';
    }

    public function up(Schema $schema): void
    {
        $this->addSql(<<<'SQL'
            create materialized view IF NOT EXISTS view_chill_main_address_reference as
            SELECT row_number() OVER ()                                                AS row_number,
                   cmar.id                                                             AS address_id,
                   lower(unaccent(cmar.street || ' '::text || cmar.streetnumber || ' '::text || cmpc.code::text || ' '::text ||
                   cmpc.label::text))                                          AS address,
                   cmpc.id                                                             AS postcode_id
            FROM chill_main_address_reference cmar
                     JOIN chill_main_postal_code cmpc ON cmar.postcode_id = cmpc.id
            WHERE cmar.deletedat IS NULL
            ORDER BY ((cmpc.code::text || ' '::text) || cmpc.label::text), cmar.street, (lpad(cmar.streetnumber, 10, '0'::text));
            SQL);

        $this->addSql(<<<'SQL'
            create index if not exists view_chill_internal_address_reference_trgm
                on view_chill_main_address_reference using gist (postcode_id, address public.gist_trgm_ops);
        SQL);
    }

    public function down(Schema $schema): void
    {
        $this->addSql('DROP MATERIALIZED VIEW view_chill_main_address_reference');
    }
}
